@@ -4,34 +4,37 @@
 # Non-robust version
 tl <- function(x, ...) {
   x <- as.ts(x)
-  tspx <- tsp(x)
   n <- length(x)
   tt <- 1:n
   fit <- supsmu(tt, x)
   out <- ts(cbind(trend = fit$y, remainder = x - fit$y))
   tsp(out) <- tsp(x)
 
-  out <- structure(list(time.series = out), class = "stl")
-  return(out)
+  structure(list(time.series = out), class = "stl")
 }
 
 # Function to return some bootstrap samples of x
 # based on LPB
-lpb <- function(x, nsim=100) {
+lpb <- function(x, nsim = 100) {
   n <- length(x)
   meanx <- mean(x)
   y <- x - meanx
-  gamma <- wacf(y, lag.max = n)$acf[, , 1]
+  gamma <- wacf(y, lag.max = n)$acf[,, 1]
   s <- length(gamma)
   Gamma <- matrix(1, s, s)
   d <- row(Gamma) - col(Gamma)
-  for (i in 1:(s - 1))
+  for (i in 1:(s - 1)) {
     Gamma[d == i | d == (-i)] <- gamma[i + 1]
+  }
   L <- t(chol(Gamma))
   W <- solve(L) %*% matrix(y, ncol = 1)
-  out <- ts(L %*% matrix(sample(W, n * nsim, replace = TRUE), nrow = n, ncol = nsim) + meanx)
+  out <- ts(
+    L %*%
+      matrix(sample(W, n * nsim, replace = TRUE), nrow = n, ncol = nsim) +
+      meanx
+  )
   tsp(out) <- tsp(x)
-  return(out)
+  out
 }
 
 
@@ -42,14 +45,15 @@ MBB <- function(x, window_size) {
   bx <- array(0, (floor(length(x) / window_size) + 2) * window_size)
   for (i in 1:(floor(length(x) / window_size) + 2)) {
     c <- sample(1:(length(x) - window_size + 1), 1)
-    bx[((i - 1) * window_size + 1):(i * window_size)] <- x[c:(c + window_size - 1)]
+    bx[((i - 1) * window_size + 1):(i * window_size)] <- x[
+      c:(c + window_size - 1)
+    ]
   }
   start_from <- sample(0:(window_size - 1), 1) + 1
   bx[start_from:(start_from + length(x) - 1)]
 }
 
 
-
 #' Box-Cox and Loess-based decomposition bootstrap.
 #'
 #' Generates bootstrapped versions of a time series using the Box-Cox and
@@ -65,7 +69,7 @@ MBB <- function(x, window_size) {
 #' @return A list with bootstrapped versions of the series. The first series in
 #' the list is the original series.
 #' @author Christoph Bergmeir, Fotios Petropoulos
-#' @seealso \code{\link{baggedETS}}.
+#' @seealso [baggedETS()].
 #' @references Bergmeir, C., R. J. Hyndman, and J. M. Benitez (2016). Bagging
 #' Exponential Smoothing Methods using STL Decomposition and Box-Cox
 #' Transformation. International Journal of Forecasting 32, 303-312.
@@ -74,17 +78,18 @@ MBB <- function(x, window_size) {
 #' bootstrapped_series <- bld.mbb.bootstrap(WWWusage, 100)
 #'
 #' @export
-bld.mbb.bootstrap <- function(x, num, block_size=NULL) {
-
-  if(length(x) <= 1L)
+bld.mbb.bootstrap <- function(x, num, block_size = NULL) {
+  if (length(x) <= 1L) {
     return(rep(list(x), num))
+  }
 
   freq <- frequency(x)
-  if(length(x) <= 2*freq)
+  if (length(x) <= 2 * freq) {
     freq <- 1L
+  }
 
   if (is.null(block_size)) {
-    block_size <- ifelse(freq > 1, 2 * freq, min(8, floor(length(x) / 2)))
+    block_size <- if (freq > 1) 2 * freq else min(8, floor(length(x) / 2))
   }
 
   xs <- list()
@@ -107,9 +112,13 @@ bld.mbb.bootstrap <- function(x, num, block_size=NULL) {
       remainder <- x.stl[, 3]
     } else {
       # Loess
-      trend <- 1:length(x)
+      trend <- seq_along(x)
       suppressWarnings(
-        x.loess <- loess(ts(x.bc, frequency = 1) ~ trend, span = 6 / length(x), degree = 1)
+        x.loess <- loess(
+          ts(x.bc, frequency = 1) ~ trend,
+          span = 6 / length(x),
+          degree = 1
+        )
       )
       seasonal <- rep(0, length(x))
       trend <- x.loess$fitted
@@ -119,7 +128,10 @@ bld.mbb.bootstrap <- function(x, num, block_size=NULL) {
 
   # Bootstrap some series, using MBB
   for (i in 2:num) {
-    xs[[i]] <- ts(InvBoxCox(trend + seasonal + MBB(remainder, block_size), lambda))
+    xs[[i]] <- ts(InvBoxCox(
+      trend + seasonal + MBB(remainder, block_size),
+      lambda
+    ))
     tsp(xs[[i]]) <- tsp(x)
   }
 
