@@ -4,35 +4,38 @@
 #'
 #' Innovation residuals correspond to the white noise process that drives the
 #' evolution of the time series model. Response residuals are the difference
-#' between the observations and the fitted values (equivalent to \code{h}-step
-#' forecasts). For functions with no \code{h} argument, \code{h=1}. For
+#' between the observations and the fitted values (equivalent to `h`-step
+#' forecasts). For functions with no `h` argument, `h = 1`. For
 #' homoscedastic models, the innovation residuals and the response residuals
-#' for \code{h=1} are identical. Regression residuals are available for
+#' for `h = 1` are identical. Regression residuals are available for
 #' regression models with ARIMA errors, and are equal to the original data
 #' minus the effect of the regression variables. If there are no regression
 #' variables, the errors will be identical to the original series (possibly
-#' adjusted to have zero mean).  \code{arima.errors} is a deprecated function
-#' which is identical to \code{residuals.Arima(object, type="regression")}.
-#' For \code{nnetar} objects, when \code{type="innovations"} and \code{lambda} is used, a
+#' adjusted to have zero mean). `arima.errors` is a deprecated function
+#' which is identical to `residuals.Arima(object, type="regression")`.
+#' For `nnetar` objects, when `type = "innovations"` and `lambda` is used, a
 #' matrix of time-series consisting of the residuals from each of the fitted neural networks is returned.
 #'
-#' @param object An object containing a time series model of class \code{ar},
-#' \code{Arima}, \code{bats}, \code{ets}, \code{arfima}, \code{nnetar} or
-#' \code{stlm}.
-#' If \code{object} is of class \code{forecast}, then the function will return
-#' \code{object$residuals} if it exists, otherwise it returns the differences between
+#' @param object An object containing a time series model of class `ar`,
+#' `Arima`, `bats`, `ets`, `arfima`, `nnetar` or `stlm`.
+#' If `object` is of class `forecast`, then the function will return
+#' `object$residuals` if it exists, otherwise it returns the differences between
 #' the observations and their fitted values.
 #' @param type Type of residual.
-#' @param h If \code{type='response'}, then the fitted values are computed for
-#' \code{h}-step forecasts.
+#' @param h If `type = "response"`, then the fitted values are computed for
+#' `h`-step forecasts.
 #' @param ... Other arguments not used.
-#' @return A \code{ts} object.
+#' @return A `ts` object.
 #' @author Rob J Hyndman
-#' @seealso \code{\link{fitted.Arima}}, \code{\link{checkresiduals}}.
+#' @seealso [fitted.Arima()], [checkresiduals()].
 #' @keywords ts
 #'
 #' @export
-residuals.forecast <- function(object, type=c("innovation", "response"), ...) {
+residuals.forecast <- function(
+  object,
+  type = c("innovation", "response"),
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$residuals
@@ -43,7 +46,7 @@ residuals.forecast <- function(object, type=c("innovation", "response"), ...) {
 
 #' @rdname residuals.forecast
 #' @export
-residuals.ar <- function(object, type=c("innovation", "response"), ...) {
+residuals.ar <- function(object, type = c("innovation", "response"), ...) {
   type <- match.arg(type)
   # innovation and response residuals are the same for AR models
   object$resid
@@ -53,12 +56,17 @@ residuals.ar <- function(object, type=c("innovation", "response"), ...) {
 #'
 #' @aliases residuals.forecast_ARIMA
 #' @examples
-#' fit <- Arima(lynx,order=c(4,0,0), lambda=0.5)
+#' fit <- Arima(lynx, order = c(4, 0, 0), lambda = 0.5)
 #'
 #' plot(residuals(fit))
-#' plot(residuals(fit, type='response'))
+#' plot(residuals(fit, type = "response"))
 #' @export
-residuals.Arima <- function(object, type=c("innovation", "response", "regression"), h=1, ...) {
+residuals.Arima <- function(
+  object,
+  type = c("innovation", "response", "regression"),
+  h = 1,
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$residuals
@@ -71,7 +79,7 @@ residuals.Arima <- function(object, type=c("innovation", "response", "regression
     }
     xreg <- getxreg(object)
     # Remove intercept
-    if (is.element("intercept", names(object$coef))) {
+    if ("intercept" %in% names(object$coef)) {
       xreg <- cbind(rep(1, length(x)), xreg)
     }
     # Return errors
@@ -80,8 +88,11 @@ residuals.Arima <- function(object, type=c("innovation", "response", "regression
     } else {
       norder <- sum(object$arma[1:4])
       return(ts(
-        c(x - xreg %*% as.matrix(object$coef[(norder + 1):length(object$coef)])),
-        frequency = frequency(x), start = start(x)
+        c(
+          x - xreg %*% as.matrix(object$coef[(norder + 1):length(object$coef)])
+        ),
+        frequency = frequency(x),
+        start = start(x)
       ))
     }
   }
@@ -92,7 +103,12 @@ residuals.forecast_ARIMA <- residuals.Arima
 
 #' @rdname residuals.forecast
 #' @export
-residuals.bats <- function(object, type=c("innovation", "response"), h=1, ...) {
+residuals.bats <- function(
+  object,
+  type = c("innovation", "response"),
+  h = 1,
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$errors
@@ -103,7 +119,12 @@ residuals.bats <- function(object, type=c("innovation", "response"), h=1, ...) {
 
 #' @rdname residuals.forecast
 #' @export
-residuals.tbats <- function(object, type=c("innovation", "response"), h=1, ...) {
+residuals.tbats <- function(
+  object,
+  type = c("innovation", "response"),
+  h = 1,
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$errors
@@ -114,7 +135,12 @@ residuals.tbats <- function(object, type=c("innovation", "response"), h=1, ...)
 
 #' @rdname residuals.forecast
 #' @export
-residuals.ets <- function(object, type=c("innovation", "response"), h=1, ...) {
+residuals.ets <- function(
+  object,
+  type = c("innovation", "response"),
+  h = 1,
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$residuals
@@ -125,14 +151,15 @@ residuals.ets <- function(object, type=c("innovation", "response"), h=1, ...) {
 
 #' @rdname residuals.forecast
 #' @export
-residuals.ARFIMA <- function(object, type=c("innovation", "response"), ...) {
+residuals.ARFIMA <- function(object, type = c("innovation", "response"), ...) {
   type <- match.arg(type)
   if (type == "innovation") {
-    if (!is.null(object$residuals)) { # Object produced by arfima()
+    if (!is.null(object$residuals)) {
+      # Object produced by arfima()
       return(object$residuals)
-    } else # Object produced by fracdiff()
-    {
-      if (is.element("x", names(object))) {
+    } else {
+      # Object produced by fracdiff()
+      if ("x" %in% names(object)) {
         x <- object$x
       } else {
         x <- eval.parent(parse(text = as.character(object$call)[2]))
@@ -141,38 +168,49 @@ residuals.ARFIMA <- function(object, type=c("innovation", "response"), ...) {
         x <- BoxCox(x, object$lambda)
       }
       y <- fracdiff::diffseries(x - mean(x), d = object$d)
-      fit <- arima(y, order = c(length(object$ar), 0, length(object$ma)), include.mean = FALSE, fixed = c(object$ar, -object$ma))
+      fit <- arima(
+        y,
+        order = c(length(object$ar), 0, length(object$ma)),
+        include.mean = FALSE,
+        fixed = c(object$ar, -object$ma)
+      )
       return(residuals(fit, type = "innovation"))
     }
-  }
-  else {
+  } else {
     getResponse(object) - fitted(object)
   }
 }
 
 #' @rdname residuals.forecast
 #' @export
-residuals.nnetar <- function(object, type=c("innovation", "response"), h=1, ...) {
+residuals.nnetar <- function(
+  object,
+  type = c("innovation", "response"),
+  h = 1,
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation" && !is.null(object$lambda)) {
-    res <- matrix(unlist(lapply(object$model, residuals)), ncol = length(object$model))
+    res <- matrix(
+      unlist(lapply(object$model, residuals)),
+      ncol = length(object$model)
+    )
     if (!is.null(object$scalex$scale)) {
       res <- res * object$scalex$scale
     }
-  }
-  else {
+  } else {
     res <- getResponse(object) - fitted(object, h = h)
   }
 
   tspx <- tsp(getResponse(object))
   res <- ts(res, frequency = tspx[3L], end = tspx[2L])
 
-  return(res)
+  res
 }
 
 #' @rdname residuals.forecast
 #' @export
-residuals.stlm <- function(object, type=c("innovation", "response"), ...) {
+residuals.stlm <- function(object, type = c("innovation", "response"), ...) {
   type <- match.arg(type)
   if (type == "innovation") {
     object$residuals
@@ -183,7 +221,11 @@ residuals.stlm <- function(object, type=c("innovation", "response"), ...) {
 
 #' @rdname residuals.forecast
 #' @export
-residuals.tslm <- function(object, type=c("innovation", "response", "deviance"), ...) {
+residuals.tslm <- function(
+  object,
+  type = c("innovation", "response", "deviance"),
+  ...
+) {
   type <- match.arg(type)
   if (type == "innovation" || type == "deviance") {
     object$residuals
